/*:ts=8*/
/*****************************************************************************
 * FIDOGATE --- Gateway UNIX Mail/News <-> FTN NetMail/EchoMail
 *
 * $Id: ftninpost.c,v 1.1.2.2 2002/11/30 00:40:15 dyff Exp $
 *
 * Processing inbound packets
 *
 *****************************************************************************
 * Copyright (C) 2001
 * 
 *    Dmitry Fedotov            FIDO:      2:5030/1229
 *				Internet:  dyff@fidogate.spb.ru
 *
 * This file is part of FIDOGATE.
 *
 * FIDOGATE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * FIDOGATE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with FIDOGATE; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include "fidogate.h"
#include "getopt.h"


#define PROGRAM 	"ftninpost"
#define VERSION 	"$Revision: 1.1.2.2 $"
#define CONFIG		DEFAULT_CONFIG_MAIN



/*
 * Prototypes
 */


void	short_usage		(void);
void	usage			(void);


int	do_dir			(char *, int);

char	ftnin_sendmail[MAXPATH] = {0};


/*
 * Usage messages
 */
void short_usage(void)
{
    fprintf(stderr, "usage: %s [-options]\n", PROGRAM);
    fprintf(stderr, "       %s --help  for more information\n", PROGRAM);
}


void usage(void)
{
    fprintf(stderr, "FIDOGATE %s  %s %s\n\n",
	    version_global(), PROGRAM, version_local(VERSION) );
    
    fprintf(stderr, "usage:   %s [-options]\n\n", PROGRAM);
    fprintf(stderr, "\
options: -v --verbose                 more verbose\n\
	 -h --help                    this help\n\
         -c --config name             read config file (\"\" = none)\n");
    
    exit(0);
}



int do_dir(char *cdir, int mail)
{
    DIR *dp;
    struct dirent *dir;
    char buf[MAXPATH+MAXINETADDR];

    if( ! ( dp = opendir(cdir) ) )
    {
	debug(7,"can't open directory %s", cdir);
	return ERROR;
    }
    while((dir = readdir(dp)))
    {
	if(wildmatch(dir->d_name, "*.rfc", TRUE))
	{
	    debug(8, "Processing %s/%s", cdir, dir->d_name);
	    if(mail)
	    {
		char rfc_file[MAXPATH];
		FILE *fp, *fp1;
		char *p, *s;

		BUF_COPY3(rfc_file, cdir, "/", dir->d_name);
		fp = fopen(rfc_file, "r");
		if(!fp)
		    continue;
		p = fgets(buffer, sizeof(buffer), fp);
		if(!p)
		    continue;
		if(strncmp(p, "From ", 5))
		    debug(9, "WARNING: \"From\" string not found");

		s = xstrtok(p+5?p+5:"", " \t");
		BUF_COPY(buf, ftnin_sendmail);
		if( (p = strstr(buf, " -f%s")) )
		    BUF_COPY(p, p+5);
		BUF_APPEND2(buf, " -f", s);

		if( fseek(fp, 0, SEEK_SET) == ERROR )
		    log("$WARNING: can't seek to offset 0 in file %s", rfc_file);
		debug(8, "exec: %s", buf);
	        fp1 = popen(buf, W_MODE);
		if(!fp1)
		{
		    log("ERROR: can't open pipe to %s", buf);
		    return ERROR;
		}
		while(cf_getline(buffer, BUFFERSIZE, fp))
		{
		    fprintf(fp1, "%s", buffer);
		}
		pclose(fp1);
		fclose(fp);

	    }
	    else
	    {
		run_system(ftnin_rnews);
	    }
	}
    }
    dir_close();

    return OK;
}


/***** main() ****************************************************************/

int main(int argc, char **argv)
{
    int c;
    char *c_flag=NULL;
    char *ftnin_rnews;
    char buf[MAXPATH] = {0};
    char *p;

    int option_index;
    static struct option long_options[] =
    {
	{ "verbose",      0, 0, 'v'},	/* More verbose */
	{ "help",         0, 0, 'h'},	/* Help */
	{ "config",       1, 0, 'c'},	/* Config file */
	{ 0,              0, 0, 0  }
    };

    log_program(PROGRAM);
    
    /* Init configuration */
    cf_initialize();


    while ((c = getopt_long(argc, argv, "vhc:",
			    long_options, &option_index     )) != EOF)
	switch (c) {
	/***** Common options *****/
	case 'v':
	    verbose++;
	    break;
	case 'h':
	    usage();
	    exit(0);
	    break;
	case 'c':
	    c_flag = optarg;
	    break;
	default:
	    short_usage();
	    exit(EX_USAGE);
	    break;
	}

    /*
     * Read config file
     */
    cf_read_config_file(c_flag ? c_flag : CONFIG);

    cf_debug();

    passwd_init();

    ftnin_rnews = NULL;

    if( (p = cf_get_string("FTNInSendmail", TRUE)) )
    {
	debug(8, "config: FTNInSendmail %s", p);
	BUF_COPY(ftnin_sendmail, p);
    }
    else
	debug(9, "WARNING: FTNInSendmail not defined in %s", c_flag ? c_flag : CONFIG);

    if( (p = cf_get_string("FTNInRnews", TRUE)) )
    {
	debug(8, "config: FTNInRnews %s", p);
	ftnin_rnews = p;
    }
    else
	debug(9, "WARNING: FTNInRnews not defined in %s", c_flag ? c_flag : CONFIG);

    if( (p = cf_get_string("FTNInRecombine", TRUE)) )
    {
	debug(8, "config: FTNInRecombine %s", p);
	BUF_EXPAND(buf, p);
    }
    else
	debug(9, "WARNING: FTNInRecombine not defined in %s", c_flag ? c_flag : CONFIG);

    if(*buf)
	run_system(buf);

    BUF_EXPAND(buf, cf_p_outrfc_mail());
    do_dir(buf, TRUE);
    BUF_EXPAND(buf, cf_p_outrfc_news());
    do_dir(buf, FALSE);
    

    exit_free();
    exit(EX_OK);
}
